@@ -3,7 +3,7 @@
 
   Resets statistics gathered by pg_qualstats.
 */
-CREATE FUNCTION @extschema@.pg_qualstats_reset()
+CREATE FUNCTION pg_qualstats_reset()
 RETURNS void
 AS 'MODULE_PATHNAME'
 LANGUAGE C;
@@ -13,7 +13,7 @@ LANGUAGE C;
 
   Returns an example for a normalized query, given its queryid
 */
-CREATE FUNCTION @extschema@.pg_qualstats_example_query(bigint)
+CREATE FUNCTION pg_qualstats_example_query(bigint)
 RETURNS text
 AS 'MODULE_PATHNAME'
 LANGUAGE C;
@@ -24,7 +24,7 @@ LANGUAGE C;
 
   Returns all the example queries with their associated queryid
 */
-CREATE FUNCTION @extschema@.pg_qualstats_example_queries(OUT queryid bigint, OUT query text)
+CREATE FUNCTION pg_qualstats_example_queries(OUT queryid bigint, OUT query text)
 RETURNS SETOF record
 AS 'MODULE_PATHNAME'
 LANGUAGE C;
@@ -129,7 +129,7 @@ LANGUAGE C;
       constvalue                | 2::integer
       eval_type                 | f
 */
-CREATE FUNCTION @extschema@.pg_qualstats(
+CREATE FUNCTION pg_qualstats(
   OUT userid oid,
   OUT dbid oid,
   OUT lrelid oid,
@@ -181,7 +181,7 @@ LANGUAGE C STRICT VOLATILE;
     opname (text):
       the name of the operator. Corresponds to opno.
 */
-CREATE FUNCTION @extschema@.pg_qualstats_names(
+CREATE FUNCTION pg_qualstats_names(
   OUT userid oid,
   OUT dbid oid,
   OUT lrelid oid,
@@ -227,16 +227,16 @@ LANGUAGE C STRICT VOLATILE;
 
   This view is just a simple wrapper on the :func:`pg_qualstats()` function, filtering on the current database for convenience.
 */
-CREATE VIEW @extschema@.pg_qualstats AS
-  SELECT qs.* FROM @extschema@.pg_qualstats() qs
+CREATE VIEW pg_qualstats AS
+  SELECT qs.* FROM pg_qualstats() qs
   INNER JOIN pg_database on qs.dbid = pg_database.oid
   WHERE pg_database.datname = current_database();
 
 
-GRANT SELECT ON @extschema@.pg_qualstats TO PUBLIC;
+GRANT SELECT ON pg_qualstats TO PUBLIC;
 
 -- Don't want this to be available to non-superusers.
-REVOKE ALL ON FUNCTION @extschema@.pg_qualstats_reset() FROM PUBLIC;
+REVOKE ALL ON FUNCTION pg_qualstats_reset() FROM PUBLIC;
 
 /*"""
 .. view:: pg_qualstats_pretty
@@ -263,7 +263,7 @@ REVOKE ALL ON FUNCTION @extschema@.pg_qualstats_reset() FROM PUBLIC;
     nbfiltered (bigint):
       the total number of tuples filtered by this qual.
 */
-CREATE VIEW @extschema@.pg_qualstats_pretty AS
+CREATE VIEW pg_qualstats_pretty AS
   select
         nl.nspname as left_schema,
         al.attrelid::regclass as left_table,
@@ -275,7 +275,7 @@ CREATE VIEW @extschema@.pg_qualstats_pretty AS
         sum(occurences) as occurences,
         sum(execution_count) as execution_count,
         sum(nbfiltered) as nbfiltered
-  from @extschema@.pg_qualstats qs
+  from pg_qualstats qs
   left join (pg_class cl inner join pg_namespace nl on nl.oid = cl.relnamespace) on cl.oid = qs.lrelid
   left join (pg_class cr inner join pg_namespace nr on nr.oid = cr.relnamespace) on cr.oid = qs.rrelid
   left join pg_attribute al on al.attrelid = qs.lrelid and al.attnum = qs.lattnum
@@ -284,7 +284,7 @@ CREATE VIEW @extschema@.pg_qualstats_pretty AS
 ;
 
 
-CREATE OR REPLACE VIEW @extschema@.pg_qualstats_all AS
+CREATE OR REPLACE VIEW pg_qualstats_all AS
   SELECT dbid, relid, userid, queryid, array_agg(distinct attnum) as attnums,
     opno, max(qualid) as qualid, sum(occurences) as occurences,
     sum(execution_count) as execution_count, sum(nbfiltered) as nbfiltered,
@@ -306,7 +306,7 @@ CREATE OR REPLACE VIEW @extschema@.pg_qualstats_all AS
           qs.execution_count as execution_count,
           qs.nbfiltered as nbfiltered,
           qs.queryid
-    FROM @extschema@.pg_qualstats() qs
+    FROM pg_qualstats() qs
     WHERE lrelid IS NOT NULL or rrelid IS NOT NULL
   ) t GROUP BY dbid, relid, userid, queryid, opno, coalesce(qualid, qualnodeid)
 ;
@@ -325,7 +325,7 @@ CREATE OR REPLACE VIEW @extschema@.pg_qualstats_all AS
     eval_type (char):
       the evaluation type. See :func:`pg_qualstats()` for an explanation of the eval_type.
 */
-CREATE TYPE @extschema@.qual AS (
+CREATE TYPE qual AS (
   relid oid,
   attnum integer,
   opno oid,
@@ -348,21 +348,16 @@ CREATE TYPE @extschema@.qual AS (
     eval_type (char):
       the evaluation type. See :func:`pg_qualstats()` for an explanation of the eval_type.
 */
-CREATE TYPE @extschema@.qualname AS (
+CREATE TYPE qualname AS (
   relname text,
   attnname text,
   opname text,
   eval_type "char"
 );
 
-CREATE TYPE @extschema@.adv_quals AS (
-    qualnodeids bigint[],
-    queryids bigint[]
-);
-
-CREATE OR REPLACE VIEW @extschema@.pg_qualstats_by_query AS
+CREATE OR REPLACE VIEW pg_qualstats_by_query AS
         SELECT coalesce(uniquequalid, uniquequalnodeid) as uniquequalnodeid, dbid, userid,  coalesce(qualid, qualnodeid) as qualnodeid, occurences, execution_count, nbfiltered, queryid,
-      array_agg(constvalue order by constant_position) as constvalues, array_agg(ROW(relid, attnum, opno, eval_type)::@extschema@.qual) as quals
+      array_agg(constvalue order by constant_position) as constvalues, array_agg(ROW(relid, attnum, opno, eval_type)::qual) as quals
       FROM
       (
 
@@ -387,12 +382,12 @@ CREATE OR REPLACE VIEW @extschema@.pg_qualstats_by_query AS
             qs.nbfiltered as nbfiltered,
             qs.eval_type,
             qs.constant_position
-        FROM @extschema@.pg_qualstats() qs
+        FROM pg_qualstats() qs
         WHERE (qs.lrelid IS NULL) != (qs.rrelid IS NULL)
     ) i GROUP BY coalesce(uniquequalid, uniquequalnodeid), coalesce(qualid, qualnodeid),  dbid, userid, occurences, execution_count, nbfiltered, queryid
 ;
 
-CREATE OR REPLACE FUNCTION @extschema@.pg_qualstats_deparse_qual(qual qual) RETURNS TEXT
+CREATE OR REPLACE FUNCTION pg_qualstats_deparse_qual(qual qual) RETURNS TEXT
 AS $_$
     SELECT pg_catalog.format('%I.%I %s ?',
         c.oid::regclass, a.attname, o.oprname)
@@ -403,18 +398,18 @@ AS $_$
     AND a.attnum = qual.attnum
 $_$ LANGUAGE sql;
 
-CREATE OR REPLACE FUNCTION @extschema@.pg_qualstats_get_qualnode_rel(bigint)
+CREATE OR REPLACE FUNCTION pg_qualstats_get_qualnode_rel(bigint)
 RETURNS TEXT
 AS $_$
     SELECT pg_catalog.quote_ident(n.nspname) || '.'
       || pg_catalog.quote_ident(c.relname)
-    FROM @extschema@.pg_qualstats() q
+    FROM pg_qualstats() q
     JOIN pg_catalog.pg_class c ON coalesce(q.lrelid, q.rrelid) = c.oid
     JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
     WHERE q.qualnodeid = $1
 $_$ LANGUAGE sql;
 
-CREATE OR REPLACE FUNCTION @extschema@.pg_qualstats_get_idx_col(bigint,
+CREATE OR REPLACE FUNCTION pg_qualstats_get_idx_col(bigint,
     include_nondefault_opclass boolean = true)
 RETURNS TEXT
 AS $_$
@@ -426,7 +421,7 @@ AS $_$
     ELSE
     ''
     END
-    FROM @extschema@.pg_qualstats() q
+    FROM pg_qualstats() q
     JOIN pg_catalog.pg_class c ON coalesce(q.lrelid, q.rrelid) = c.oid
     JOIN pg_catalog.pg_attribute a ON a.attrelid = c.oid
       AND a.attnum = coalesce(q.lattnum, q.rattnum)
@@ -440,27 +435,28 @@ AS $_$
     ORDER BY CASE opcdefault WHEN true THEN 0 ELSE 1 END;
 $_$ LANGUAGE sql;
 
-CREATE OR REPLACE FUNCTION @extschema@.pg_qualstats_index_advisor (
+CREATE OR REPLACE FUNCTION pg_qualstats_index_advisor (
     min_filter integer DEFAULT 1000,
     min_selectivity integer DEFAULT 30,
     forbidden_am text[] DEFAULT '{}')
     RETURNS json
 AS $_$
 DECLARE
+    v_res json;
     v_processed bigint[] = '{}';
-    v_indexes json[] = '{}';
-    v_unoptimised json[] = '{}';
+    v_indexes text[] = '{}';
+    v_unoptimised text[] = '{}';
 
     rec record;
     v_nb_processed integer = 1;
 
     v_ddl text;
     v_col text;
+    v_cur json;
     v_qualnodeid bigint;
     v_quals_todo bigint[];
     v_quals_done bigint[];
     v_quals_col_done text[];
-    v_queryids bigint[] = '{}';
 BEGIN
     -- sanity checks and default values
     SELECT coalesce(min_filter, 1000), coalesce(min_selectivity, 30),
@@ -473,16 +469,11 @@ BEGIN
         forbidden_am := array_append(forbidden_am, 'hash');
     END IF;
 
-    -- first find out unoptimizable quals.
-    -- We need an array of json containing the per-qual info, and a single
-    -- array containing all the underlying qualnodeids, so we need to create
-    -- the wanted final object manually as we can't have two different grouping
-    -- approach.
-    FOR rec IN WITH src AS (SELECT DISTINCT qualnodeid,
+    -- first find out unoptimizable quals
+    FOR rec IN SELECT DISTINCT qualnodeid,
         (coalesce(lrelid, rrelid), coalesce(lattnum, rattnum),
-          opno, eval_type)::@extschema@.qual AS qual,
-          queryid
-      FROM @extschema@.pg_qualstats() q
+          opno, eval_type)::qual AS qual
+      FROM pg_qualstats() q
       JOIN pg_catalog.pg_database d ON q.dbid = d.oid
       LEFT JOIN pg_catalog.pg_operator op ON op.oid = q.opno
       LEFT JOIN pg_catalog.pg_amop amop ON amop.amopopr = op.oid
@@ -491,20 +482,10 @@ BEGIN
        AND eval_type = 'f'
        AND coalesce(lrelid, rrelid) != 0
        AND amname IS NULL
-    )
-    SELECT pg_catalog.json_build_object(
-            'qual', @extschema@.pg_qualstats_deparse_qual(qual),
-            -- be careful to generate an empty array if no queryid availiable
-            'queryids',
-            coalesce(pg_catalog.array_agg(DISTINCT queryid)
-                FILTER (WHERE queryid IS NOT NULL), '{}')
-        ) AS obj,
-        array_agg(qualnodeid) AS qualnodeids
-    FROM src
-    GROUP BY qual
     LOOP
-        v_unoptimised := array_append(v_unoptimised, rec.obj);
-        v_processed := array_cat(v_processed, rec.qualnodeids);
+      v_unoptimised := pg_catalog.array_append(v_unoptimised,
+        pg_qualstats_deparse_qual(rec.qual));
+      v_processed := pg_catalog.array_append(v_processed, rec.qualnodeid);
     END LOOP;
 
     -- The index suggestion is done in multiple iteration, by scoring for each
@@ -521,7 +502,7 @@ BEGIN
         WITH pgqs AS (
           SELECT dbid, amname, qualid, qualnodeid,
             (coalesce(lrelid, rrelid), coalesce(lattnum, rattnum),
-            opno, eval_type)::@extschema@.qual AS qual, queryid,
+            opno, eval_type)::qual AS qual,
             round(avg(execution_count)) AS execution_count,
             sum(occurences) AS occurences,
             round(sum(nbfiltered)::numeric / sum(occurences)) AS avg_filter,
@@ -529,7 +510,7 @@ BEGIN
               THEN 0
               ELSE round(sum(nbfiltered::numeric) / sum(execution_count) * 100)
             END AS avg_selectivity
-          FROM @extschema@.pg_qualstats() q
+          FROM pg_qualstats() q
           JOIN pg_catalog.pg_database d ON q.dbid = d.oid
           JOIN pg_catalog.pg_operator op ON op.oid = q.opno
           JOIN pg_catalog.pg_amop amop ON amop.amopopr = op.oid
@@ -540,15 +521,12 @@ BEGIN
           AND coalesce(lrelid, rrelid) != 0
           AND qualnodeid != ALL(v_processed)
           GROUP BY dbid, amname, qualid, qualnodeid, lrelid, rrelid,
-            lattnum, rattnum, opno, eval_type, queryid
+            lattnum, rattnum, opno, eval_type
         ),
         -- apply cardinality and selectivity restrictions
         filtered AS (
           SELECT (qual).relid, amname, coalesce(qualid, qualnodeid) AS parent,
-            count(*) AS weight,
-            (array_agg(DISTINCT qualnodeid),
-             array_agg(queryid)
-            )::@extschema@.adv_quals AS quals
+            count(*) AS weight, array_agg(qualnodeid) AS quals
           FROM pgqs
           WHERE avg_filter >= min_filter
           AND avg_selectivity >= min_selectivity
@@ -559,28 +537,24 @@ BEGIN
           SELECT p.relid, p.amname, p.parent, p.quals,
             c.quals AS children
           FROM filtered p
-          LEFT JOIN filtered c ON (p.quals).qualnodeids @> (c.quals).qualnodeids
+          LEFT JOIN filtered c ON p.quals @> c.quals
             AND p.amname = c.amname
             AND p.parent != c.parent
-            AND (p.quals).qualnodeids != (c.quals).qualnodeids
+            AND p.quals != c.quals
         ),
         -- build the "paths", which is the list of AND-ed quals that entirely
         -- contains another possibly AND-ed quals, and give a score for each
         -- path.  The scoring method used here is simply the number of
         -- columns in the quals.
         paths AS (
-          SELECT DISTINCT *,
-            coalesce(pg_catalog.array_length((children).qualnodeids, 1),
-                     0) AS weight
+          SELECT DISTINCT *, coalesce(pg_catalog.array_length(children, 1), 0) AS weight
           FROM nodes
           UNION
           SELECT DISTINCT p.relid, p.amname, p.parent, p.quals, c.children,
-            coalesce(pg_catalog.array_length((c.children).qualnodeids, 1),
-                     0) AS weight
+            coalesce(pg_catalog.array_length(c.children, 1), 0) AS weight
           FROM nodes p
-          JOIN nodes c ON (p.children).qualnodeids @> (c.quals).qualnodeids
-            AND (c.quals).qualnodeids IS NOT NULL
-            AND (c.quals).qualnodeids != (p.quals).qualnodeids
+          JOIN nodes c ON p.children @> c.quals AND c.quals IS NOT NULL
+            AND c.quals != p.quals
             AND p.amname = c.amname
         ),
         -- compute the final paths.
@@ -595,8 +569,7 @@ BEGIN
           SELECT relid, amname, parent, quals,
             array_agg(to_json(children) ORDER BY weight)
               FILTER (WHERE children IS NOT NULL) AS included,
-            pg_catalog.array_length((quals).qualnodeids, 1)
-                + sum(weight) AS path_weight,
+            pg_catalog.array_length(quals, 1) + sum(weight) AS path_weight,
           CASE amname WHEN 'btree' THEN 1 ELSE 2 END AS amweight
           FROM paths
           GROUP BY relid, amname, parent, quals
@@ -610,9 +583,7 @@ BEGIN
           FROM computed
         )
         -- and finally choose the higher rank final path for each relation.
-        SELECT relid, amname, parent,
-            (quals).qualnodeids as quals, (quals).queryids as queryids,
-            included, path_weight
+        SELECT relid, amname, parent, quals, included, path_weight
         FROM final
         WHERE rownum = 1
       LOOP
@@ -624,23 +595,15 @@ BEGIN
         v_quals_col_done := '{}';
 
         -- put columns from included quals, if any, first for order dependency
-        DECLARE
-            v_cur json;
-        BEGIN
-            IF rec.included IS NOT NULL THEN
-              FOR v_cur IN SELECT v->'qualnodeids'
-                    FROM (SELECT * FROM unnest(rec.included)) AS r(v)
-                    ORDER BY pg_catalog.json_array_length(v->'qualnodeids') ASC
-              LOOP
-                -- Direct cast from json to bigint is only possible since pg10
-                FOR v_qualnodeid IN
-                    SELECT pg_catalog.json_array_elements(v_cur)::text::bigint
-                LOOP
-                  v_quals_todo := v_quals_todo || v_qualnodeid;
-                END LOOP;
-              END LOOP;
-            END IF;
-        END;
+        IF rec.included IS NOT NULL THEN
+          FOREACH v_cur IN ARRAY rec.included LOOP
+            -- Direct cast from json to bigint is only possible since pg10
+            FOR v_qualnodeid IN SELECT pg_catalog.json_array_elements(v_cur)::text::bigint
+            LOOP
+              v_quals_todo := v_quals_todo || v_qualnodeid;
+            END LOOP;
+          END LOOP;
+        END IF;
 
         -- and append qual's own columns
         v_quals_todo := v_quals_todo || rec.quals;
@@ -651,7 +614,7 @@ BEGIN
           CONTINUE WHEN v_quals_done @> ARRAY[v_qualnodeid];
 
           -- skip other quals for the same column
-          v_col := @extschema@.pg_qualstats_get_idx_col(v_qualnodeid, false);
+          v_col := pg_qualstats_get_idx_col(v_qualnodeid, false);
           CONTINUE WHEN v_quals_col_done @> ARRAY[v_col];
 
           -- mark this qual as present in a generated index so it's ignore at
@@ -667,7 +630,7 @@ BEGIN
 
           -- append the column to the index
           IF v_ddl != '' THEN v_ddl := v_ddl || ', '; END IF;
-          v_ddl := v_ddl || @extschema@.pg_qualstats_get_idx_col(v_qualnodeid, true);
+          v_ddl := v_ddl || pg_qualstats_get_idx_col(v_qualnodeid, true);
         END LOOP;
 
         -- if underlying table has been dropped, skip this (broken) index
@@ -675,47 +638,16 @@ BEGIN
 
         -- generate the full CREATE INDEX ddl
         v_ddl = pg_catalog.format('CREATE INDEX ON %s USING %I (%s)',
-          @extschema@.pg_qualstats_get_qualnode_rel(v_qualnodeid), rec.amname, v_ddl);
+          pg_qualstats_get_qualnode_rel(v_qualnodeid), rec.amname, v_ddl);
 
-        -- get the underlyings queryid(s)
-        DECLARE
-            v_queryid text;
-            v_cur json;
-        BEGIN
-            v_queryids = rec.queryids;
-            IF rec.included IS NOT NULL THEN
-              FOREACH v_cur IN ARRAY rec.included LOOP
-                -- Direct cast from json to bigint is only possible since pg10
-                FOR v_queryid IN SELECT pg_catalog.json_array_elements(v_cur->'queryids')::text
-                LOOP
-                  CONTINUE WHEN v_queryid = 'null';
-                  v_queryids := v_queryids || v_queryid::text::bigint;
-                END LOOP;
-              END LOOP;
-            END IF;
-        END;
-
-        -- remove any duplicates
-        SELECT pg_catalog.array_agg(DISTINCT v) INTO v_queryids
-            FROM (SELECT unnest(v_queryids)) s(v);
-
-        -- sanitize the queryids
-        IF v_queryids IS NULL OR v_queryids = '{null}' THEN
-            v_queryids = '{}';
-        END IF;
-
-        -- and finally append the index to the list of generated indexes
-        v_indexes := pg_catalog.array_append(v_indexes,
-            pg_catalog.json_build_object(
-                'ddl', v_ddl,
-                'queryids', v_queryids
-            )
-        );
+        -- and append it to the list of generated indexes
+        v_indexes := array_append(v_indexes, v_ddl);
       END LOOP;
     END LOOP;
 
-    RETURN pg_catalog.json_build_object(
-        'indexes', v_indexes,
+    v_res := pg_catalog.json_build_object('indexes', v_indexes,
         'unoptimised', v_unoptimised);
+
+    RETURN v_res;
 END;
-$_$ LANGUAGE plpgsql;       /* end of pg_qualstats_index_advisor */
+$_$ LANGUAGE plpgsql;
