@@ -20,7 +20,7 @@
  * The implementation is heavily inspired by pg_stat_statements
  *
  * Copyright (c) 2014,2017 Ronan Dunklau
- * Copyright (c) 2018-2024, The Powa-Team
+ * Copyright (c) 2018-2025, The Powa-Team
  *-------------------------------------------------------------------------
  */
 #include <limits.h>
@@ -812,9 +812,9 @@ pgqs_ExecutorEnd(QueryDesc *queryDesc)
 		if (context->nentries)
 		{
 			/*
-			 * Before acquiring exlusive lwlock, check if there's enough room
+			 * Before acquiring exclusive lwlock, check if there's enough room
 			 * to store local hash.  Also, do not remove more than 20% of
-			 * maximum number of entries in shared memory (wether they are
+			 * maximum number of entries in shared memory (whether they are
 			 * used or not). This should not happen since we shouldn't store
 			 * that much entries in localhash in the first place.
 			 */
@@ -2391,6 +2391,17 @@ pgqs_resolve_var(Var *var, pgqsWalkerContext *context)
 	List	   *tlist = NULL;
 	PlanState  *planstate = context->planstate;
 
+	/*
+	 * This function can recurse, so this can prevent infinite loop in case of
+	 * any problem.
+	 */
+	CHECK_FOR_INTERRUPTS();
+
+#if PG_VERSION_NUM >= 140000
+	/* ROWID_VAR is only used during planning, so we shouldn't see it. */
+	Assert(var->varno != ROWID_VAR);
+#endif
+
 	pgqs_set_planstates(context->planstate, context);
 	switch (var->varno)
 	{
@@ -2437,6 +2448,13 @@ pgqs_resolve_var(Var *var, pgqsWalkerContext *context)
 
 	pgqs_set_planstates(planstate, context);
 
+	/*
+	 * If the resolved var is still a var with a special varno, we need to
+	 * resolve it again.
+	 */
+	if (IsA(var, Var) && IS_SPECIAL_VARNO(var->varno))
+		return pgqs_resolve_var(var, context);
+
 	return (Expr *) var;
 }
 
